package com.rizla.android;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;

import org.json.JSONException;
import org.json.JSONObject;

import com.rizla.android.R;

import android.app.Activity;
import android.content.Context;
import android.graphics.Typeface;
import android.os.Handler;
import android.util.Log;
import android.view.View;
import android.view.ViewGroup;
import android.webkit.JavascriptInterface;
import android.webkit.WebView;
import android.widget.LinearLayout;
import android.widget.TextView;

public class RizlaWebInterface {
	String title;
	private static Context mContext;
	private Handler mHandler;
	private static WebView mWebView;
	private LinearLayout mBodyLayout;
	private ArrayList<Element> elements;
	private ArrayList<Element> list_item;
	private ArrayList<Element> listview_content_item;
	
    /** Instantiate the interface and set the context */
    public RizlaWebInterface(Context c, WebView wv) {
        mContext = c;
        mWebView = wv;
        
        this.mHandler = new Handler();
        this.mBodyLayout = (LinearLayout)((Activity) mContext).findViewById(R.id.rlayout);
        this.elements = new ArrayList<Element>();
        this.list_item = new ArrayList<Element>();
        this.listview_content_item = new ArrayList<Element>();
    }
    
    /*************************************
     * ELEMENT CREATION
     ************************************/
    @JavascriptInterface
    public void createImage(final int id, final String url, final int parent, String style) { 	
		RImage ri = new RImage(mContext,mWebView,id,url);
		this.elements.add(new Element(parent, ri));
    }
    
    @JavascriptInterface
    public void createButton(final int id, final String title, final int parent, String style) { 	
		RButton rb = new RButton(mContext,mWebView,id,title);
    	/* button style like dialog*/
		//RButton rb = new RButton(mContext,null,android.R.attr.buttonBarButtonStyle,mWebView,id,title);
		RizlaCssEngine rce = new RizlaCssEngine(style);
		rce.applyStyle(rb);
		this.elements.add(new Element(parent, rb));
    }
    
    @JavascriptInterface
    public void createLink(final int id, final String title, final String url, final int parent) {
		RLink b = new RLink(mContext,null,android.R.attr.buttonBarButtonStyle,mWebView,id,title,url);
		this.elements.add(new Element(parent, b));
    }
    
    @JavascriptInterface
    public void createTextView(final int id, final String title, final int parent, String style) { 
		RTextView tv = new RTextView(mContext,mWebView,id,title);
		RizlaCssEngine rce = new RizlaCssEngine(style);
		rce.applyStyle(tv);
		constructCreationList(tv,parent);
    }
    
    @JavascriptInterface
    public void createStrong(final int id, final String title, final int parent, String style) { 	
		RTextView tv = new RTextView(mContext,mWebView,id,title);
		tv.setTypeface(null, Typeface.BOLD);
		RizlaCssEngine rce = new RizlaCssEngine(style);
		rce.applyStyle(tv);
		constructCreationList(tv,parent);
    }
    
    @JavascriptInterface
    public void createH1(final int id, final String title, final int parent, String style) { 	
		RTextView tv = new RTextView(mContext,mWebView,id,title);
		tv.setTypeface(null, Typeface.BOLD);
		tv.setTextSize(50);
		RizlaCssEngine rce = new RizlaCssEngine(style);
		rce.applyStyle(tv);
		constructCreationList(tv,parent);
    }
    
    @JavascriptInterface
    public void createH2(final int id, final String title, final int parent, String style) { 	
		RTextView tv = new RTextView(mContext,mWebView,id,title);
		tv.setTypeface(null, Typeface.BOLD);
		tv.setTextSize(40);
		RizlaCssEngine rce = new RizlaCssEngine(style);
		rce.applyStyle(tv);
		constructCreationList(tv,parent);
    }
    
    @JavascriptInterface
    public void createH3(final int id, final String title, final int parent, String style) { 	
		RTextView tv = new RTextView(mContext,mWebView,id,title);
		tv.setTypeface(null, Typeface.BOLD);
		tv.setTextSize(30);
		RizlaCssEngine rce = new RizlaCssEngine(style);
		rce.applyStyle(tv);
		constructCreationList(tv,parent);
    }
    
    @JavascriptInterface
    public void createH4(final int id, final String title, final int parent, String style) { 	
		RTextView tv = new RTextView(mContext,mWebView,id,title);
		tv.setTypeface(null, Typeface.BOLD);
		tv.setTextSize(20);
		RizlaCssEngine rce = new RizlaCssEngine(style);
		rce.applyStyle(tv);
		constructCreationList(tv,parent);
    }
    
    @JavascriptInterface
    public void createH5(final int id, final String title, final int parent, String style) { 	
		RTextView tv = new RTextView(mContext,mWebView,id,title);
		tv.setTypeface(null, Typeface.BOLD);
		tv.setTextSize(18);
		RizlaCssEngine rce = new RizlaCssEngine(style);
		rce.applyStyle(tv);
		constructCreationList(tv,parent);
    }
    
    @JavascriptInterface
    public void createH6(final int id, final String title, final int parent, String style) { 	
		RTextView tv = new RTextView(mContext,mWebView,id,title);
		tv.setTypeface(null, Typeface.BOLD);
		tv.setTextSize(14);
		RizlaCssEngine rce = new RizlaCssEngine(style);
		rce.applyStyle(tv);
		constructCreationList(tv,parent);
    }
    
    @JavascriptInterface
    public void createLayout(final int id, final String type, final int parent) {	
		RLayout ly = new RLayout(mContext,mWebView,id,type);
		this.elements.add(new Element(parent, ly));	
    }
    
    @JavascriptInterface
    public void createList(final int id, final int parent) {
    	RList rl = new RList(mContext,mWebView,id);   	
		RListAdapter ca = new RListAdapter(mContext,(ArrayList<View>) rl.getListItem(),id);
    	rl.setAdapter(ca);	
		this.elements.add(new Element(parent, rl));	
    }
    
    @JavascriptInterface
    public void createListItem(final int id, final int parent){
    	RLayout ly = new RLayout(mContext,mWebView,id,"vertical");
		this.list_item.add(new Element(parent, ly));
    }
    
    @JavascriptInterface
    public void createBody(final int id, String style) {
		this.mBodyLayout.setId(id);
    }
    
    @JavascriptInterface
    public void addElements(){
    	
    	mHandler.post(new Runnable() {
    		public void run() {
    			Log.d("RUN","IM RUNNIG");
		    	for(Element e : elements){
		    		Log.d("parentId",String.valueOf(e.parentId));
		    		ViewGroup parentView = (ViewGroup)((Activity) mContext).findViewById(e.parentId);
		    		parentView.addView(e.element);
		    	}
		    	elements.clear();
		    	/*listview element added at end on the process*/
		    	for(Element e : list_item){
		    		RList rl = (RList)((Activity) mContext).findViewById(e.parentId);
		    		
		    		for(Element lc : listview_content_item){
		    			if (lc.parentId == e.element.getId()){
		    				((ViewGroup) e.element).addView(lc.element);
		    			}
		    		}
		    		
		    		rl.addItem(e.element);
		    		((RListAdapter) rl.getAdapter()).notifyDataSetChanged();
		    	}
    		}
    	});
    }
    
    /*determine if a child is attached on a list*/
    public void constructCreationList(View child, int parent){
    	boolean isListItem = false;
		if (list_item.size() > 0) {
			for (Element e : list_item){
				if (e.element.getId() == parent){
					this.listview_content_item.add(new Element(parent,child));
					isListItem = true;
					break;
				}
				
			}
			if (!isListItem)
				this.elements.add(new Element(parent, child));
		} else {
			this.elements.add(new Element(parent, child));
		}
    }
    
    /*************************************
     * UPDATE ELEMENTS
     ************************************/
    @JavascriptInterface
    public void updateTextView(final int id, final String title){
    	mHandler.post(new Runnable() {
    		public void run() {
		    	TextView tv = (TextView)((Activity) mContext).findViewById(id);
		    	Log.d("TV UPDATE",title);
		    	tv.setText(title);
    		}
    	});
    };
    
    /*************************************
     * PLUGINS
     ************************************/
    private static Object createPlugin(Class<?> classPlugin) throws NoSuchMethodException, ClassNotFoundException {

    		Constructor<?> constructor = classPlugin.getConstructor(new Class[]{Class.forName("android.webkit.WebView"), Class.forName("android.content.Context")});
		    Object[] arguments = new Object[]{mWebView,mContext};
    		Log.d("Constructeur","Constructor: " + constructor.toString());
		    Object object = null;

		    try {
		      object = constructor.newInstance(arguments);
		      Log.d("Object","Object: " + object.toString());
		      return object;
		    } catch (InstantiationException e) {
		      //handle it
		    } catch (IllegalAccessException e) {
		      //handle it
		    } catch (IllegalArgumentException e) {
		      //handle it
		    } catch (InvocationTargetException e) {
		      //handle it
		    }
		    return object;
	}
		

    @JavascriptInterface
    public Object plugin(String name, String object){
    	Log.d("Plugin",name);
		try {
			Class<?> c = Class.forName("com.rizla.plugins."+name);
			Plugin plug = (Plugin) createPlugin(c);
			JSONObject json = (JSONObject) new JSONObject(object);
			return plug.run(json);
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		} catch (NoSuchMethodException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (JSONException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return null;
    }
    
    
}
